static void csv_import(uint highest_id, char delim) {
	uint i;
	int end_field = 0;
	uint end_field_type = FIELD_TYPE_NORMAL;
	int current_face = IN_FRONT;
	uint new_id = highest_id + 1;

	uint line_count = 1;

	uint parse_index = 0;
	uint parse_buff_size = PARSE_BUFF_SIZE_DEFAULT;
	char *parse_buff = xmalloc(sizeof(char) * parse_buff_size);

	/* 
	 * the fields array alternates between the front face and back face.
	 * Even number indexes are front, odd are back. A pair is a record.
	 */
	uint fields_index = 0;
	uint fields_buff_size = FIELDS_BUFF_SIZE_DEFAULT;
	struct csv_field *fields = xmalloc(sizeof(struct csv_field) * fields_buff_size);

	int ch = '\0';

	for (;;) {
		ch = getchar();

		if (ch == EOF && parse_index == 0) {
			break;
		}

		if (parse_index >= parse_buff_size - 1) {
			parse_buff_size *= 2;
			parse_buff = xrealloc(parse_buff, sizeof(char) * parse_buff_size);
		}

		if (ch == delim || ch == '\n' || ch == EOF) {
			/* If it's an empty field */
			if (parse_index == 0) {
				die("ikna: Empty field on line %u\n"
					"No changes made to current deck.\n", line_count);
			}

			if (parse_buff[0] == '"') {
				if (parse_index > 1 && parse_buff[parse_index - 1] == '"') {
					end_field = 1;
					end_field_type = FIELD_TYPE_QUOTED;
				}
			} else {
				end_field = 1;
				end_field_type = FIELD_TYPE_NORMAL;
			}

			if (end_field) {
				if (current_face == IN_FRONT && ch == '\n') {
					die("ikna: Can't end first field with a newline on line %u.\n"
						"No changes made to current deck.\n", line_count);
				}

				if (current_face == IN_FRONT && ch == EOF) {
					die("ikna: Can't end first field with a EOF on line %u\n"
						"No changes made to current deck.\n", line_count);
				}

				if (current_face == IN_BACK && ch == delim) {
					die("ikna: Too many fields for record on line %u\n"
						"No changes made to current deck.\n", line_count);
				}

				if (end_field_type == FIELD_TYPE_QUOTED) {
					int hit_quote = 0;
					int bad_quote = 0;
					uint i;

					if (parse_index == 2) {
						die("ikna: Can't have quoted field with nothing in it on line %u\n"
							"No changes made to current deck.\n", line_count);
					}

					for (i = 1; i < parse_index - 1; ++i) {
						if (hit_quote && parse_buff[i] != '"') {
							bad_quote = 1;
							break;
						}

						if (parse_buff[i] == '"' && !hit_quote) {
							hit_quote = 1;
						} else {
							hit_quote = 0;
						}
					}

					if (parse_buff[i - 1] == '"' && hit_quote) {
						bad_quote = 1;
					}

					if (bad_quote) {
						die("ikna: Bad escaping of inner quotes in quoted field on line %u\n"
							"No changes made to current deck.\n", line_count);
					}
				} else if (end_field_type != FIELD_TYPE_QUOTED) {
					/* According to RFC4180 you can't have double quotes inside an unquoted field */
					uint i;

					for (i = 0; i <= parse_index; ++i) {
						if (parse_buff[i] == '"') {
							die("ikna: Double quote found in unquoted field on line %u\n"
								"No changes made to current deck.\n", line_count);
						}
					}
				}
			}
		}

		if (end_field) {
			if (fields_index >= fields_buff_size - 1) {
				fields_buff_size *= 2;
				fields = xrealloc(fields, sizeof(struct csv_field) * fields_buff_size);
			}

			/* 
			 * Interviewer: So how did you get the CSV parser to work?
			 * Tim: It's quite simple, really. I called malloc() for every field.
			 * Interviewer: Brilliant.
			 */
			fields[fields_index].type = end_field_type;
			fields[fields_index].content = xmalloc(sizeof(char) * parse_index + 1);
			memcpy(fields[fields_index].content, parse_buff, parse_index + 1);

			if (current_face == IN_FRONT) {
				current_face = IN_BACK;
			} else if (current_face == IN_BACK) {
				current_face = IN_FRONT;
			}

			end_field_type = FIELD_TYPE_NORMAL;
			end_field = 0;
			parse_index = 0;

			++fields_index;
		} else {
			parse_buff[parse_index] = ch;
			parse_buff[parse_index + 1] = '\0';

			++parse_index;
		}

		if (ch == '\n') {
			++line_count;
		}

		if (ch == EOF) {
			break;
		}
	}

	for (i = 0; i < fields_index; i += 2) {
		/* TODO: Assumes there's at least one card in the log with ID 0 */
		struct csv_field *front, *back;
		front = &fields[i];
		back = &fields[i + 1];

		if (front->type == FIELD_TYPE_QUOTED) {
			fix_quoted_field_string(front->content);
		}

		if (back->type == FIELD_TYPE_QUOTED) {
			fix_quoted_field_string(back->content);
		}

		/* printf("%u: %s\t%s\n", new_id, front->content, back->content); */
		new_card(front->content, back->content, new_id);
		++new_id;
	}
}

static void csv_export(uint highest_id, char delim) {
	uint i;
	char *front;
	char *back;

	for (i = 0; i <= highest_id; ++i) {
		char *id_string = xmalloc(sizeof(char) * MAX_CARD_ID_BUFFER_SIZE);
		sprintf(id_string, "%u", i);
		get_card_faces(id_string, &front, &back);
		free(id_string);

		/* Don't diss it if it works */
		putchar('"');
		print_string_duplicate_double_quotes(front);
		putchar('"');
		putchar(delim);
		putchar('"');
		print_string_duplicate_double_quotes(back);
		putchar('"');
		putchar('\n');

		free(front);
		free(back);
	}
}

static void print_string_duplicate_double_quotes(char *str) {
	char *j;
	for (j = str; *j != '\0'; ++j) {
		if (*j == '"') {
			putchar('"');
		}
		putchar(*j);
	}
}

/* Edits input string! Must be \0 terminated! */
static void delete_index_from_string(char *string, uint index_to_delete) {
	uint i;
	char ch = '\0';

	i = index_to_delete + 1;
	for (;;) {
		ch = string[i];

		string[i - 1] = ch;

		if (ch == '\0') {
			break;
		}

		++i;
	}
}

/* Edits input string! Must be \0 terminated! */
static void fix_quoted_field_string(char *field_string) {
	uint i;
	int hit_quote = 0;

	uint len = strlen(field_string) + 1; /* len including null terminator */

	delete_index_from_string(field_string, len - 2);
	delete_index_from_string(field_string, 0);
	len -= 2;

	for (i = 0; i < len; ++i) {
		if (field_string[i] == '"') {
			if (hit_quote) {
				delete_index_from_string(field_string, i);
				--len;
				if (field_string[i] == '"') {
					hit_quote = 1;
				} else {
					hit_quote = 0;
				}
			} else {
				hit_quote = 1;
			}
		}
	}
}
